"use strict";
var Boom = require("boom");
var React = require("react");
var ReactDOMServer = require("react-dom/server");
var ReactRouter = require("react-router");
var lodash_1 = require("lodash");
var es6_promise_1 = require("es6-promise");
var HapiReactSSR = (function () {
    function HapiReactSSR(server, options, next) {
        var _this = this;
        this.handler = function (request, reply) {
            var _a = _this.options, routes = _a.routes, bootstrapAction = _a.bootstrapAction, template = _a.template, visionOptions = _a.visionOptions, params = _a.params, rootElement = _a.rootElement, getInitialContext = _a.getInitialContext, renderToStaticMarkup = _a.renderToStaticMarkup, contextAsProps = _a.contextAsProps;
            ReactRouter.match({ routes: routes, location: request.url }, function (error, redirectLocation, renderProps) {
                if (error) {
                    return reply(Boom.badImplementation(error.message));
                }
                else if (redirectLocation) {
                    return reply.redirect(redirectLocation.pathname + redirectLocation.search);
                }
                else if (!renderProps) {
                    return reply(Boom.notFound("Invalid route", { route: request.url.pathname }));
                }
                var context = getInitialContext();
                es6_promise_1.Promise.all(renderProps.components.filter(function (component) { return component[bootstrapAction]; })
                    .map(function (component) { return component[bootstrapAction](context, renderProps); })).then(function (response) {
                    var reactRootElement = React.createElement(rootElement, contextAsProps ? context : { context: context }, React.createElement(ReactRouter.RouterContext, renderProps));
                    var renderContext = lodash_1.assign(params, { context: JSON.stringify(context) }, {
                        componentRenderedToString: renderToStaticMarkup ?
                            ReactDOMServer.renderToStaticMarkup(reactRootElement) : ReactDOMServer.renderToString(reactRootElement)
                    });
                    return reply.view(template, renderContext, visionOptions);
                }).catch(function (err) {
                    return reply(Boom.badImplementation(err.message));
                });
            });
        };
        this.options = options;
        server.route({
            path: "/{path*}",
            method: "GET",
            handler: this.handler
        });
        next();
    }
    HapiReactSSR.attributes = {
        pkg: require("../package.json"),
        dependencies: "vision"
    };
    return HapiReactSSR;
}());
exports.register = HapiReactSSR;
